% Users must cite the following paper:
% 1) Kim, Shaver, and Funk. Forthcoming. "From Mass to Motion: Conceptualizing and Measuring the Dynamics of Industry Clusters," Strategic Management Journal

clc;
clear;
tic();

data = csvread('pop_year_cbsa.csv',1,0); % Input file #1: Total number of population in a year-region 

starty = min(data(:,1));
endy = max(data(:,1));

year_seq = starty:endy;
[nrow ncol] = size(data);

prob = NaN(nrow/length(year_seq),length(year_seq));
for y = 1:length(year_seq)
    datatemp = data((data(:,1) == year_seq(y)),:);
%     disp(size(datatemp,1))
    prob(:,y) = datatemp(:,3)/sum(datatemp(:,3));
end

cbsalist = datatemp(:,2);
n_cbsa = size(cbsalist,1);
S = 100;    % The number of repeats
data_est = csvread('est_semiconductor_year.csv',1,0); % Input file #2: Total number of the focal industry establishments in a year
nfirm = data_est(:,2);

for y = 1:length(year_seq)
    nsample = nfirm(y);
    rec = zeros(n_cbsa,S);
    for s = 1:S
        rdsam = randsample(n_cbsa,nsample,1, prob(:,y));
        for i = 1:n_cbsa 
            rec(i,s) = sum(rdsam(:) == i);
        end
    end
    mean_res = mean(rec,2);
    std_res = std(rec,1,2);
    Outmat = [ones(n_cbsa,1)*year_seq(y),cbsalist, mean_res,std_res];
    
    Outname = ['MC_output_' num2str(year_seq(y)) '.txt'];
    fileID = fopen(Outname,'w');
    fprintf(fileID,'%s,%s,%s,%s\n','year','CBSA','mean','SD');
    fprintf(fileID,'%4d,%5d,%.2f,%.16f\n',Outmat');
    fclose(fileID);
end


df = [];
for i=starty:endy  
    filestr = ['MC_output_',num2str(i),'.txt'];
    data = readtable(filestr);
    df = [df; data];
end

datacombine = xlsread('est_semiconductor_year_cbsa35620.xlsx');  % Input file #3: Total number of the focal industry establishments in a year-region
% title = df.Properties.VariableNames;
df.Properties.VariableNames = {'year', 'CBSA', 'mean', 'SD'} ;
% datacombine(:,[3:7 9:end]) = [];        % remove irrelevant columns, if the datacombine has only 3 columns, we do not need this line.
datacombine = array2table(datacombine);
datacombine.Properties.VariableNames(1:3) = {'year', 'CBSA', 'est'} ;


nrow = size(df,1);
df(:,5) = array2table(zeros(nrow,1));
for i=1:nrow
    index = find(datacombine.year == table2array(df(i,1)) & datacombine.CBSA == table2array(df(i,2)));
    if isempty(index) == 0
        df(i,5) = datacombine(index,3);
%         disp(i)
    end 
end


cbsa_index = table2array(unique(datacombine(:,2)));
dfout = table2array(df);
dfout(:,6) = (dfout(:,5) - dfout(:,3))./dfout(:,4);
dfout = dfout(ismember(dfout(:,2),cbsa_index),:);
dfout(:,3:5) = [];

% ncol = n_cbsa;
ncol = size(cbsa_index,1);
dataout = reshape(dfout(:,3), ncol,(endy-starty+1))';

header =  cellstr(num2str(dfout(1:ncol,2)));
for i = 1:length(header)
    header(i) = cellstr(['zscore_cbsa',char(header(i))]);
end

year = (starty:endy)';

Out = [cellstr('year') header';num2cell(year) num2cell(dataout)];
%xlswrite('Clevel_output_BPtest_input.xlsx',Out)
writetable(table(Out), 'Clevel_output_BPtest_input_semiconductor.xlsx','WriteVariableNames',false)


toc();